//! Overriding an exception handler
//!
//! You can override an exception handler using the [`#[exception]`][1] attribute.
//!
//! [1]: https://rust-embedded.github.io/cortex-m-rt/0.6.1/cortex_m_rt_macros/fn.exception.html
//!
//! ---

#![no_main]
#![no_std]

use panic_halt as _;

use cortex_m::interrupt::free;
use cortex_m::peripheral::NVIC;
use cortex_m_rt::entry;
use cortex_m_semihosting::{debug, hprintln};
use lm3s6965::{interrupt, Interrupt};
use rtic_0::{self, init2, App, ConvertNr, Task};

const T1: Task = Task {
    interrupt: ConvertNr(Interrupt::UART0.nr()), // UART0
    prio: 1,
};

const T2: Task = Task {
    interrupt: ConvertNr(Interrupt::UART1.nr()), // UART1
    prio: 2,
};

const TASKS: [Task; 2] = [T1, T2];

const APP: App = App {
    nvic_priority_bits: lm3s6965::NVIC_PRIO_BITS,
    tasks: &TASKS,
};

#[entry]
fn main() -> ! {
    init2(&APP);

    // test to trigger UART0
    // unsafe { NVIC::unmask(Interrupt::UART0) };
    free(|_cs| {
        // let t = Interrupt::UART0.nr();
        NVIC::pend(T1.interrupt);
        NVIC::pend(T2.interrupt);
        // let t = Interrupt::UART1.nr();
        // NVIC::pend(ConvertNr(Interrupt::UART1.nr()));
    });
    // stop
    debug::exit(debug::EXIT_SUCCESS);
    loop {}
}

#[interrupt]
fn UART0() {
    hprintln!("UART0").ok();
}

#[interrupt]
fn UART1() {
    hprintln!("UART1").ok();
}
