#![no_std]

// use core::ptr::{self, Unique};
use cortex_m::interrupt::InterruptNumber;
use cortex_m::peripheral::NVIC;
use cortex_m_semihosting::hprintln;
pub struct Task {
    pub interrupt: ConvertNr,
    pub prio: u8, // logic priority
}

pub struct App {
    pub nvic_priority_bits: u8,
    pub tasks: &'static [Task],
}

#[derive(Copy, Clone)]
pub struct ConvertNr(pub u8);

unsafe impl InterruptNumber for ConvertNr {
    fn number(self) -> u16 {
        self.0 as u16
    }
}

pub fn init(tasks: &[&dyn cortex_m::interrupt::Nr]) {
    hprintln!("init").ok();
    for t in tasks {
        unsafe {
            let tmp = ConvertNr(t.nr());
            NVIC::unmask(tmp);
        }
    }
}

pub fn init2(app: &App) {
    hprintln!("init2").ok();
    for t in app.tasks {
        unsafe {
            NVIC::unmask(t.interrupt);
            let mut p = cortex_m::peripheral::Peripherals::steal();
            p.NVIC
                .set_priority(t.interrupt, logical2hw(t.prio, app.nvic_priority_bits));
        }
    }
}

fn logical2hw(priority: u8, prio_bits: u8) -> u8 {
    // the NVIC encodes priority in the higher bits of a bit
    // also a bigger numbers means lower priority
    ((1 << prio_bits) - priority) << (8 - prio_bits)
}

// core.NVIC.set_priority(Interrupt::UART0, logical2hw(uart0_prio));
